﻿
' ===================================================================
'  DATA FLOW
' ===================================================================
' File "Configuration.txt"
' Sub  "Load_Configuration"
'    Global parameters to : gButtonWidth, gButtonHeight, gFormColor, gBackColor, gTextColor, gTextFont, gTextSize, gTextBold
'    Single parameters to : Array "ActiveObjects" As "ActiveObject"
'
' Sub  "InitActiveObjects"
'    If ActiveObjects(i) is button
'       INITIALIZE Button
'    Else
'       ActiveSlots.AddActiveSlot
'
' ButtonClickHandler
'    ActiveSlots.AddActiveSlot


Module ActiveObjects

    Friend Structure ActiveObject
        Dim ArrayIndex As Int32
        Dim MousePressed As Boolean
        Dim Button As MyButton
        Dim TextBox As System.Windows.Forms.TextBox
        Dim ActiveSlotControl As MyActiveSlotControl
        Dim Type As ActiveObjectType
        Dim MoveType As String
        Dim SlotString As String
        Dim SlotArray() As Int32
        Dim Value As Single
        Dim pos As Single
        Dim Speed As Single
        Dim Random As Single
        Dim accumulator As Single
        Dim SpeedCounter As Single
        Dim MultiplierSlot As Int32
        Dim TriggerSlot As Int32
        Dim PulseTimer As Int32
        Dim Text As String
        Dim TextAlign As HorizontalAlignment
        Dim ContentAlign As ContentAlignment
        Dim TextFont As Font
        Dim TextColor As Color
        Dim BackColor As Color
        Dim Left As Int32
        Dim Top As Int32
        Dim Width As Int32
        Dim Height As Int32
    End Structure

    Friend Enum ActiveObjectType
        Button
        TextBox
        ActiveSlot
    End Enum

    Friend ActiveObjectsArray(-1) As ActiveObject

    Friend gFormColor As Color = Color.White
    Friend gTextFont As Font = New Font("Arial", 12)
    Friend gTextColor As Color = Color.Black
    Friend gBackColor As Color = Color.White
    Friend gTextAlign As String = "Center"
    Friend gWidth As Int32 = 120
    Friend gHeight As Int32 = 90


    ' =============================================================================
    '  ACTIVE OBJECTS - INIT
    ' =============================================================================
    Friend Sub ClearActiveObjects()
        Form1.FlowLayoutPanel1.Controls.Clear()
        ReDim ActiveObjectsArray(-1)
    End Sub

    Friend Sub InitActiveObjects()
        For i As Int32 = 0 To ActiveObjectsArray.Length - 1
            Dim obj As ActiveObject = ActiveObjectsArray(i)
            Select Case obj.Type
                Case ActiveObjectType.Button
                    obj.Button = New MyButton
                    Form1.Controls.Add(obj.Button)
                    With obj.Button
                        .Parent = Form1.FlowLayoutPanel1
                        .Tag = i.ToString
                        .FlatStyle = FlatStyle.Flat
                        '.Margin = New Padding(3, 3, 3, 3)
                        '.FlatAppearance.BorderColor = Color.DarkGreen
                        '.FlatAppearance.BorderSize = 0
                        .Text = obj.Text
                        .TextAlign = obj.ContentAlign
                        .Font = obj.TextFont
                        .ForeColor = obj.TextColor
                        .BackColor = obj.BackColor
                        .Left = obj.Left
                        .Top = obj.Top
                        .Size = New Size(obj.Width, obj.Height)
                        AddHandler .Click, AddressOf ClickHandler
                        AddHandler .PointerDown, AddressOf MouseDownHandler
                        AddHandler .MouseDown, AddressOf MouseDownHandler
                        AddHandler .MouseUp, AddressOf MouseUpHandler
                        AddHandler .MouseMove, AddressOf MouseMoveHandler
                    End With
                Case ActiveObjectType.TextBox
                    obj.TextBox = New System.Windows.Forms.TextBox
                    obj.TextBox.Text = obj.Text
                    Form1.Controls.Add(obj.TextBox)
                    With obj.TextBox
                        .TabStop = False
                        .Parent = Form1.FlowLayoutPanel1
                        .Tag = i.ToString
                        '.Margin = New Padding(3, 3, 3, 3)
                        .Multiline = True
                        .ReadOnly = True
                        .Text = obj.Text
                        .TextAlign = obj.TextAlign
                        .Font = obj.TextFont
                        .ForeColor = obj.TextColor
                        .BackColor = obj.BackColor
                        .Left = obj.Left
                        .Top = obj.Top
                        .Size = New Size(obj.Width, obj.Height)
                        AddHandler .Click, AddressOf ClickHandler
                        AddHandler .MouseDown, AddressOf MouseDownHandler
                        AddHandler .MouseUp, AddressOf MouseUpHandler
                        AddHandler .MouseMove, AddressOf MouseMoveHandler
                    End With
                Case ActiveObjectType.ActiveSlot
                    obj.ActiveSlotControl = New MyActiveSlotControl
                    Form1.Controls.Add(obj.ActiveSlotControl)
                    With obj.ActiveSlotControl
                        .TabStop = False
                        .Parent = Form1.FlowLayoutPanel1
                        .Tag = i.ToString
                        .FlatStyle = FlatStyle.Flat
                        '.Margin = New Padding(3, 3, 3, 3)
                        .Text = obj.Text
                        .TextAlign = obj.ContentAlign
                        .Font = obj.TextFont
                        .ForeColor = obj.TextColor
                        .BackColor = obj.BackColor
                        .Left = obj.Left
                        .Top = obj.Top
                        .Size = New Size(obj.Width, obj.Height)
                        AddHandler .Click, AddressOf ClickHandler
                        AddHandler .MouseDown, AddressOf MouseDownHandler
                        AddHandler .MouseUp, AddressOf MouseUpHandler
                        AddHandler .MouseMove, AddressOf MouseMoveHandler
                    End With
                    ActiveSlots.AddActiveSlot(obj)
            End Select
            obj.ArrayIndex = i
            ActiveObjectsArray(i) = obj
        Next
    End Sub


    ' =============================================================================
    '  ACTIVE OBJECTS - CLICK
    ' =============================================================================
    Friend CopyPropertyFlag As Boolean
    Friend CopyHilightColor As Color = Color.OrangeRed

    Private Sub ClickHandler(ByVal sender As Object, ByVal e As EventArgs)
        Dim i As Int32 = GetActiveObjectIndex(sender)
        ' ---------------------------------------------------------- change obj props
        If CopyPropertyFlag Then
            Form2.CopyProperty(i)
            Return
        End If
        ' ---------------------------------------------------------- start editing
        If My.Computer.Keyboard.CtrlKeyDown Or Form2.Visible Then
            Form2.StartEditing(i)
        End If
    End Sub

    Private Function GetActiveObjectIndex(ByVal ctrl As Object) As Int32
        Dim n As Int32
        If TypeOf ctrl Is Button Then
            Dim b As System.Windows.Forms.Button = CType(ctrl, System.Windows.Forms.Button)
            n = CInt(Val(b.Tag))
        ElseIf TypeOf ctrl Is TextBox Then
            Dim t As System.Windows.Forms.TextBox = CType(ctrl, System.Windows.Forms.TextBox)
            n = CInt(Val(t.Tag))
        ElseIf TypeOf ctrl Is MyActiveSlotControl Then
            Dim t As MyActiveSlotControl = CType(ctrl, MyActiveSlotControl)
            n = CInt(Val(t.Tag))
        End If
        Return n
    End Function


    ' =============================================================================
    '  DRAG CONTROLS
    ' =============================================================================
    Private Dragging As Boolean
    Private StartMousePos As Point
    Private StartControlPos As Point
    Private Sub MouseDownHandler(ByVal sender As Object, ByVal e As MouseEventArgs)
        If Form2.Visible Then
            If Not Form1.AutoLayout And Not CopyPropertyFlag Then
                Dim c As Control = CType(sender, Control)
                c.BringToFront()
                StartControlPos = c.Location
                StartMousePos = Cursor.Position
                Dragging = True
                Form2.HideSelectedObjBorder()
            End If
        End If
        ' ---------------------------------------------------------- exclude TextBox and MyActiveSlotLabel
        If TypeOf sender Is TextBox Or _
           TypeOf sender Is MyActiveSlotControl Then Return
        ' ---------------------------------------------------------- add active slot 
        Dim i As Int32 = GetActiveObjectIndex(sender)
        Dim obj As ActiveObject = ActiveObjectsArray(i)
        obj.MousePressed = True
        ActiveSlots.AddActiveSlot(obj)
        If obj.Speed = 0 And obj.Random = 0 Then
            For j As Int32 = 0 To obj.SlotArray.Length - 1
                Slots.WriteSlot(obj.SlotArray(j), obj.Value)
            Next
        End If
    End Sub
    Private Sub MouseUpHandler(ByVal sender As Object, ByVal e As MouseEventArgs)
        If Dragging Then
            Dragging = False
            Form2.CopyPlacementFromControlsToActiveObjectsArray()
            Save_Configuration()
        End If
        ' ---------------------------------------------------------- update active slot with MousePressed = false
        Dim i As Int32 = GetActiveObjectIndex(sender)
        Dim obj As ActiveObject = ActiveObjectsArray(i)
        obj.MousePressed = False
        ActiveSlots.AddActiveSlot(obj)
    End Sub
    Private Sub MouseMoveHandler(ByVal sender As Object, ByVal e As MouseEventArgs)
        If dragging Then
            Dim c As Control = CType(sender, Control)
            c.Left = startControlPos.X + Cursor.Position.X - startMousePos.X
            c.Top = startControlPos.Y + Cursor.Position.Y - startMousePos.Y
        End If
    End Sub

End Module



